@@ -56,8 +56,8 @@ USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "windowstr.h"
 #include "shadow.h"
-
 #include "xaarop.h"
+#include "fb.h"
 
 #include "intel.h"
 #include "i830_reg.h"
@@ -71,7 +71,11 @@ typedef struct {
 	PixmapPtr pixmap;
 } I830DRI2BufferPrivateRec, *I830DRI2BufferPrivatePtr;
 
+#if HAS_DEVPRIVATEKEYREC
 static DevPrivateKeyRec i830_client_key;
+#else
+static int i830_client_key;
+#endif
 
 static uint32_t pixmap_flink(PixmapPtr pixmap)
 {
@@ -705,7 +709,11 @@ i830_dri2_register_frame_event_resource_types(void)
 static XID
 get_client_id(ClientPtr client)
 {
+#if HAS_DIXREGISTERPRIVATEKEY
 	XID *ptr = dixGetPrivateAddr(&client->devPrivates, &i830_client_key);
+#else
+	XID *ptr = dixLookupPrivate(&client->devPrivates, &i830_client_key);
+#endif
 	if (*ptr == 0)
 		*ptr = FakeClientID(client->index);
 	return *ptr;
@@ -1480,8 +1488,13 @@ Bool I830DRI2ScreenInit(ScreenPtr screen)
 		return FALSE;
 	}
 
+#if HAS_DIXREGISTERPRIVATEKEY
 	if (!dixRegisterPrivateKey(&i830_client_key, PRIVATE_CLIENT, sizeof(XID)))
 		return FALSE;
+#else
+	if (!dixRequestPrivate(&i830_client_key, sizeof(XID)))
+		return FALSE;
+#endif
 
 
 #if DRI2INFOREC_VERSION >= 4
